from __future__ import absolute_import
from builtins import range
from .technique import PositionInvariantTechnique
import random

class dropoutAugmentationTechnique(PositionInvariantTechnique):

    # percentage of pixels to dropout is a value between 0 and 1
    def __init__(self,parameters):
        PositionInvariantTechnique.__init__(self, parameters)
        if 'percentage' in list(parameters.keys()):
            self.percentage = float(parameters["percentage"])
        else:
            self.percentage = 0.05

    def apply(self, image):
        channels = len(image.shape)
        elems =[(x, y) for x in range(0, image.shape[0]) for y in range(0, image.shape[1])]
        random.shuffle(elems)
        dropoutelems = elems[0:int(self.percentage*len(elems))]
        imageC = image.copy()
        for dropoutelem in dropoutelems:
            if channels==3:
                imageC[dropoutelem[0],dropoutelem[1]] = [0,0,0]
            else:
                imageC[dropoutelem[0], dropoutelem[1]] = 0
        return imageC

# technique = dropoutAugmentationTechnique(0.5)
# image = cv2.imread("LPR1.jpg")
# cv2.imshow("t",technique.applyForClassification(image))
# cv2.waitKey(0)



from __future__ import division
from past.utils import old_div
import cv2
import numpy as np
import math


def generateMosaic (images):

    imagemosaic = []
    maxheight = max([i.shape[0] for i in images])
    maxwidth = max([i.shape[1] for i in images])

    rows = int(math.ceil(old_div(len(images),4.0)))
    columns = 4
    mosaicImage = np.zeros((rows*maxheight,columns*maxwidth,3),dtype="uint8")
    for (i,image) in enumerate(images):
        mosaicImage[int(old_div(i,4))*maxheight:(int(old_div(i,4))*maxheight)+image.shape[0],
                    int(i%4)*maxwidth:(int(i%4)*maxwidth)+image.shape[1]] = image
    return mosaicImage



















from __future__ import absolute_import
from . import augment
from . import generate_sample
import argparse
import sys

def main():
    arg1 = sys.argv[1]
    augment.main(arg1)

def main_sample():
    arg1 = sys.argv[1]
    arg2 = sys.argv[2]
    generate_sample.main(arg1,arg2)



from __future__ import absolute_import
from __future__ import division
from past.utils import old_div
from .technique import PositionInvariantTechnique
import cv2
import numpy as np

class gammaCorrectionAugmentationTechnique(PositionInvariantTechnique):

    # Valid values for gamma are in the range (0,2.5]
    def __init__(self,parameters):
        PositionInvariantTechnique.__init__(self, parameters)
        if 'gamma' in list(parameters.keys()):
            self.gamma = float(parameters["gamma"])
        else:
            self.gamma = 1.5

        if (self.gamma<=0 or self.gamma >2.5):
            raise NameError("Invalid value for gamma")

    def apply(self, image):
        invGamma = old_div(1.0, self.gamma)
        table = np.array([((old_div(i, 255.0)) ** invGamma) * 255
                          for i in np.arange(0, 256)]).astype("uint8")

        # apply gamma correction using the lookup table
        return cv2.LUT(image, table)


from __future__ import absolute_import
from builtins import str
from builtins import range
from builtins import object
from .iaugmentor import IAugmentor
from imutils import paths
from .utils.aspectawarepreprocessor import AspectAwarePreprocessor
import os
import cv2
import numpy as np
import random
from sklearn.preprocessing import LabelBinarizer


def readAndGenerateImage(image,label, generators):
    newimage = image
    newlabel = label
    for (j, generator) in enumerate(generators):
        if (random.randint(0,100)>50):
            newimage,newlabel = generator.applyForClassification(newimage,newlabel)

    return (newimage,newlabel)

# This class serves to generate images for a classification
# problem where all the images are organized by folders
# distributed by labels. Example:
# - Folder
# |- cats
#    |- image1.jpg
#    |- image2.jpg
#    |- ...
# |- dogs
#    |- image1.jpg
#    |- image2.jpg
#    |- ...
class FolderKerasLinearClassificationAugmentor(object):

    def __init__(self,inputPath,parameters):
        IAugmentor.__init__(self)
        self.inputPath = inputPath
        # output path represents the folder where the images will be stored
        if parameters["batchSize"]:
            self.batchSize = parameters["batchSize"]
        else:
            self.batchSize = 32
        if parameters["width"]:
            self.width = parameters["width"]
        else:
            self.width = 64
        if parameters["height"]:
            self.height = parameters["height"]
        else:
            self.height = 64
        self.generators = []
        self.readImagesAndAnnotations()

    def addGenerator(self, generator):
        self.generators.append(generator)

    def readImagesAndAnnotations(self):
        self.imagePaths = list(paths.list_files(self.inputPath,validExts=(".jpg", ".jpeg", ".png", ".bmp",".tiff",".tif")))
        random.shuffle(self.imagePaths)
        self.numImages = len(self.imagePaths)
        self.labels = [p.split(os.path.sep)[-2] for p in self.imagePaths]
        self.classes = len([str(x) for x in np.unique(self.labels)])
        le = LabelBinarizer()
        self.labels = le.fit_transform(self.labels)


    def applyAugmentation(self,passes=np.inf):
        epochs = 0
        aap = AspectAwarePreprocessor(self.width,self.height)
        batch_features = np.zeros((self.batchSize, self.width, self.height, 3))
        batch_labels = np.zeros((self.batchSize, self.classes))
        while epochs < passes:

            for i in np.arange(0, self.numImages, self.batchSize):
                imagPaths = self.imagePaths[i:i+self.batchSize]
                labels = self.labels[i:i+self.batchSize]
                images = [cv2.imread(imagePath) for imagePath in imagPaths]
                imagesLabels = [readAndGenerateImage(image,self.generators) for image in images]
                labels = [imageLabel[1] for imageLabel in imagesLabels]
                images = [aap.preprocess(imageLabel[0]) for imageLabel in imagesLabels]
                for j in range(self.batchSize):

                    index = random.randint(0,len(images)-1)
                    batch_features[j] = images[index]
                    batch_labels[j] = labels[index]
                yield (batch_features,batch_labels)


            epochs += 1












import h5py
import os
import sys

class HDF5DatasetWriterClassification:

    def __init__(self,dims,outputPath,dataKey="images",bufSize=1000):

        if os.path.exists(outputPath):
            raise ValueError("The output path already exists")

        self.db = h5py.File(outputPath,"w")
        self.data = self.db.create_dataset(dataKey,dims,dtype="float")
        self.labels = self.db.create_dataset("labels",(dims[0],),dtype="int")
        self.bufSize = bufSize
        self.buffer = {"data":[],"labels":[]}
        self.idx = 0

    def add(self,rows,labels):
        self.buffer["data"].extend(rows)
        self.buffer["labels"].extend(labels)
        if len(self.buffer["data"]) >= self.bufSize:
            self.flush()

    def flush(self):
        i = self.idx + len(self.buffer["data"])
        self.data[self.idx:i] = self.buffer["data"]
        self.labels[self.idx:i] = self.buffer["labels"]
        self.idx = i
        self.buffer = {"data": [], "labels":[]}

    def storeClassLabels(self,classLabels):
        if sys.version_info[0]<3:
            dt = h5py.special_dtype(vlen=unicode)
        else:
            dt = h5py.special_dtype(vlen=str)
        labelSet = self.db.create_dataset("label_names",
                                          (len(classLabels),),dtype=dt)
        labelSet[:] = classLabels

    def close(self):
        if len(self.buffer["data"]) > 0:
            self.flush()
        self.db.close()


class HDF5DatasetWriterSegmentation:

    def __init__(self,dims,outputPath,dataKey="images",bufSize=1000):

        if os.path.exists(outputPath):
            raise ValueError("The output path already exists")

        self.db = h5py.File(outputPath,"w")
        self.data = self.db.create_dataset(dataKey,dims,dtype="float")
        self.labels = self.db.create_dataset("labels",dims,dtype="float")
        self.bufSize = bufSize
        self.buffer = {"data":[],"labels":[]}
        self.idx = 0

    def add(self,rows,labels):
        self.buffer["data"].extend(rows)
        self.buffer["labels"].extend(labels)
        if len(self.buffer["data"]) >= self.bufSize:
            self.flush()

    def flush(self):
        i = self.idx + len(self.buffer["data"])
        self.data[self.idx:i] = self.buffer["data"]
        self.labels[self.idx:i] = self.buffer["labels"]
        self.idx = i
        self.buffer = {"data": [], "labels":[]}

    def close(self):
        if len(self.buffer["data"]) > 0:
            self.flush()
        self.db.close()


class HDF5DatasetWriterLocalization:

    def __init__(self,dims,outputPath,dataKey="images",bufSize=1000):

        if os.path.exists(outputPath):
            raise ValueError("The output path already exists")

        self.db = h5py.File(outputPath,"w")
        self.data = self.db.create_dataset(dataKey,dims,dtype="float")
        self.labels = self.db.create_dataset("labels",(dims[0],5),dtype="float")
        self.bufSize = bufSize
        self.buffer = {"data":[],"labels":[]}
        self.idx = 0

    def add(self,rows,labels):
        self.buffer["data"].extend(rows)
        self.buffer["labels"].extend(labels)
        if len(self.buffer["data"]) >= self.bufSize:
            self.flush()

    def flush(self):
        i = self.idx + len(self.buffer["data"])
        self.data[self.idx:i] = self.buffer["data"]
        self.labels[self.idx:i] = self.buffer["labels"]
        self.idx = i
        self.buffer = {"data": [], "labels":[]}

    def close(self):
        if len(self.buffer["data"]) > 0:
            self.flush()
        self.db.close()


from __future__ import absolute_import
from .technique import PositionInvariantTechnique
import cv2

class noneAugmentationTechnique(PositionInvariantTechnique):

    # Valid values for kernel are 3,5,7,9, and 11
    def __init__(self,parameters=None):
        PositionInvariantTechnique.__init__(self, parameters)


    def apply(self, image):
        return image.copy()


from __future__ import absolute_import
from .techniqueList import  Techniques
from .technique import DecoratorTechnique

def createTechnique(technique,parameters,dictLabels=None):
    Technique = Techniques[technique]
    if Technique is None:
        raise ValueError("That technique is not available")
    return DecoratorTechnique(Technique(parameters),dictLabels)


from __future__ import absolute_import
from .technique import PositionVariantTechnique
import cv2

class flipAugmentationTechnique(PositionVariantTechnique):

    # Valid values for flip are -1,0,1
    def __init__(self,parameters):
        PositionVariantTechnique.__init__(self, parameters)
        if 'flip' in list(parameters.keys()):
            self.flip = int(parameters["flip"])
        else:
            self.flip = 1
        if (not(self.flip in [-1,1,0])):
            raise NameError("Invalid value for flip")

    def apply(self, image):
        flipped = cv2.flip(image, self.flip)
        return flipped


# Example
# technique = flipAugmentationTechnique(1)
# image = cv2.imread("LPR1.jpg")
# box = ('dog',(0,0,20,20))
# imageC=technique.applyForClassification(image)
# cv2.imshow("classification",technique.applyForClassification(image))
# (imageL,(_,(x,y,w,h)))=technique.applyForLocalization(image,box)
# cv2.rectangle(imageL,(x,y),(x+w,y+h),(255,255,255),1)
# cv2.imshow("localization",imageL)
# cv2.waitKey(0)


from builtins import object
from abc import ABCMeta,abstractmethod
from future.utils import with_metaclass

class IAugmentor(with_metaclass(ABCMeta, object)):
    def addGenerator(self, generator):
        self.generators.append(generator)

    @abstractmethod
    def readImagesAndAnnotations(self):
        raise NotImplementedError

    @abstractmethod
    def applyAugmentation(self):
        raise NotImplementedError



from builtins import object
from abc import ABCMeta, abstractmethod
from future.utils import with_metaclass

class Technique(with_metaclass(ABCMeta, object)):
    def __init__(self, parameters = None):
        self.parameters = parameters

    @abstractmethod
    def apply(self, image):
        raise NotImplementedError




class PositionVariantTechnique(with_metaclass(ABCMeta, Technique)):
    def __init__(self, parameters=None):
        Technique.__init__(self, parameters)

    @abstractmethod
    def apply(self, image):
        raise NotImplementedError

class PositionInvariantTechnique(with_metaclass(ABCMeta, Technique)):
    def __init__(self, parameters=None):
        Technique.__init__(self, parameters)

    @abstractmethod
    def apply(self, image):
        raise NotImplementedError

class DecoratorTechnique(Technique):

    def __init__(self, technique,dictLabels=None):
        self.technique=technique
        self.dictLabels = dictLabels

    def apply(self,image):
        return self.technique.apply(image)

    def transform_label(self,label):
        if (self.dictLabels is not None) and (label in self.dictLabels.keys()):
            return self.dictLabels[label]
        return label



from __future__ import absolute_import
from __future__ import division
from past.utils import old_div
from .technique import PositionVariantTechnique
import cv2


class resizeAugmentationTechnique(PositionVariantTechnique):

    methods = {'INTER_NEAREST': cv2.INTER_NEAREST,
               'INTER_LINEAR': cv2.INTER_LINEAR,
               'INTER_AREA': cv2.INTER_AREA,
               'INTER_CUBIC': cv2.INTER_CUBIC,
               'INTER_LANCZOS4': cv2.INTER_LANCZOS4}

    # valid methods for interpolation methods: INTER_AREA, INTER_CUBIC,
    # INTER_NEAREST, INTER_LINEAR, INTER_LANCZOS4
    def __init__(self, parameters):
        PositionVariantTechnique.__init__(self, parameters)
        if 'percentage' in list(parameters.keys()):
            self.percentage = float(parameters["percentage"])
        else:
            self.percentage = 1.5
        if 'method' in list(parameters.keys()):
            method = parameters["method"]
        else:
            method = 'INTER_AREA'

        if (not(method in self.methods)):
            raise NameError("Invalid value for method")
        self.method = self.methods[method]

    def __resize(self,image, width=None, height=None,inter=cv2.INTER_AREA):
        # initialize the dimensions of the image to be resized and
        # grab the image size
        dim = None
        (h, w) = image.shape[:2]

        # if both the width and height are None, then return the
        # original image
        if width is None and height is None:
            return image

        # check to see if the width is None
        if width is None:
            # calculate the ratio of the height and construct the
            # dimensions
            r = old_div(height, float(h))
            dim = (int(w * r), height)

        # otherwise, the height is None
        else:
            # calculate the ratio of the width and construct the
            # dimensions
            r = old_div(width, float(w))
            dim = (width, int(h * r))

        # resize the image
        resized = cv2.resize(image, dim, interpolation=inter)

        # return the resized image
        return resized

    def apply(self, image):
        resize = self.__resize(image,width=int(image.shape[1]*self.percentage),inter=self.method)
        return resize


# technique = resizeAugmentationTechnique()
# image = cv2.imread("LPR1.jpg")
# print int(image.shape[1]*1.5)
# cv2.imshow("resized",technique.applyForClassification(image))
# cv2.waitKey(0)


from __future__ import absolute_import
from builtins import object
from .detection import detectBox,detectBoxes
from ..techniques.technique import PositionVariantTechnique,PositionInvariantTechnique,DecoratorTechnique


class Generator(object):

    def __init__(self,technique):
        if (isinstance(technique, DecoratorTechnique)):
            self.technique = technique
        else:
            raise ValueError("The technique should be an object of the class Technique")

    def applyForClassification(self, image,label):
        return (self.technique.apply(image),self.technique.transform_label(label))


    def applyForLocalization(self,image,box):
        if(isinstance(self.technique.technique,PositionVariantTechnique)):
            return [self.technique.apply(image),
                    detectBox(image,box,self.technique)]
        else:
            return [self.technique.apply(image),(self.technique.transform_label(box[0]),box[1])]

    def applyForDetection(self, image, boxes):
        if (isinstance(self.technique.technique, PositionVariantTechnique)):
            return [self.technique.apply(image),
                    detectBoxes(image, boxes, self.technique)]
        else:
            return [self.technique.apply(image),
                    [(self.technique.transform_label(box[0]),box[1]) for box in boxes]]

    def applyForSegmentation(self, image, imageLabel):
        if (isinstance(self.technique.technique, PositionVariantTechnique)):
            return [self.technique.apply(image),
                    self.technique.apply(imageLabel)]
        else:
            return [self.technique.apply(image), imageLabel]

    def applyForInstanceSegmentation(self,image,maskLabels):
        if (isinstance(self.technique.technique, PositionVariantTechnique)):
            return [self.technique.apply(image),
                    [(self.technique.apply(mask), self.technique.transform_label(label))
                     for mask,label in maskLabels ]]
        else:
            return [self.technique.apply(image),
                    [(mask, self.technique.transform_label(label))
                     for mask, label in maskLabels]]







from __future__ import absolute_import
from .technique import PositionInvariantTechnique
import cv2
import numpy as np
from keras.preprocessing.image import *

class shiftChannelAugmentationTechnique(PositionInvariantTechnique):

    # Valid values for shift are positive values
    def __init__(self,parameters):
        PositionInvariantTechnique.__init__(self, parameters)
        if 'shift' in list(parameters.keys()):
            self.shift = float(parameters["shift"])
        else:
            self.shift = 0.2


    def apply(self, image):
        if(len(image.shape)!=3):
            raise NameError("Not applicable technique")

        image1 = random_channel_shift(image, self.shift, 2)
        return image1